#!/usr/bin/ruby

require 'optparse'
require 'rexml/document'

def parse_commandline(ac)
  op = OptionParser.new
  op.banner = "usage: #{$0} -i <infile>"
  op.on('-i', '--infile=<fname>',
        'Dive profile in DM5 generated XML file format.') { |f| ac[:infile] = f }
  op.on('-h', '--help', 'Print this message.') {
    puts op.help
    exit 0
  }

  begin
    op.parse!
  rescue OptionParser::ParseError => err
    raise ArgumentError, err.message
  end
  unless ac[:infile]
    puts "No input file given."
    puts op.help
    exit 1
  end
  return ac
end

def toplevel
  app_context = {
    :infile      => nil,
    :outdir      => '.',
    :verbose     => 2
  }
  app_context = parse_commandline(app_context) # unparsed args may remain in $ARGV.
  xml = open(app_context[:infile]).read
  doc = REXML::Document.new(xml)

  doc.elements.each('Dive') { |tic|
    printf("# date: %s\n", tic.elements['StartTime'].text)
  }

  profile = []
  doc.elements.each('Dive/DiveSamples/Dive.Sample') { |tic|
    sec = tic.elements['Time'].text.to_f
    depth = tic.elements['Depth'].text.to_f
    profile.push({
                   'sec' => sec,
                   'depth' => depth,
                 })
  }

  printf("# dive time: %d s\n", profile[-1]['sec'])

  # max depth
  maxdepth = 0.0
  maxdepth_at = 0
  profile.each_with_index { |tic, i|
    if maxdepth < tic['depth']
      maxdepth = tic['depth']
      maxdepth_at = i
    end
  }
  printf("# max depth: %5.2f m at %d sec\n", maxdepth, profile[maxdepth_at]['sec']);

  # avg vels in initial 20s, 20s to bottom, bottom to surface.
  d0 = 0
  i0 = 0
  profile.each_with_index { |tic, i|
    d = tic['depth']
    if d0 < 20.0 and 20.0 < d
      printf("# [  0.00 m, % 5.2f m] avg % 5.2f m/s    time %d s\n", d, d / tic['sec'], tic['sec'])
      d0 = d
      i0 = i
    end
    if i == maxdepth_at
      dt = (tic['sec'] - profile[i0]['sec'])
      printf("# [% 5.2f m, % 5.2f m] avg % 5.2f m/s    time %d s\n", d0, d, (d - d0) / dt, dt)
             
    end
  }
  dt = profile[-1]['sec'] - profile[maxdepth_at]['sec']
  printf("# [% 5.2f m,   0.00 m] avg % 5.2f m/s    time %d s\n", maxdepth, maxdepth / dt, dt)

  printf("# time(s)    depth(m)    velocity(m/s)\n")
  profile.each_with_index { |tic, i|
    if i == 0
      vel = 0
    else
      tic0 = profile[i-1]
      vel = (tic['depth'] - tic0['depth']) / (tic['sec'] - tic0['sec'])
    end
    tic['vel'] = vel
  }
  profile.each_with_index { |tic, i|
    printf("    % 3d       %5.2f       %+5.2f\n", tic['sec'], tic['depth'], tic['vel'])
  }
end

toplevel
